'''
This script creates the endpoints for the API.

@ Author: Paul Bethge
@ Date: 01/09/2022

Copyright (c) 2022 ZKM | Karsruhe
Simplified BSD License
'''

# Imports
import os
import sys
import json
import base64
import logging
import logging.config as cnfg

from pydantic import BaseModel

from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware


from huggingsound import SpeechRecognitionModel, KenshoLMDecoder


# ==== Configuration ==== #

# path to the valid languages file
LANGS = os.path.join(os.path.dirname(os.path.realpath(__file__)), "languagecodes.json")

 # configure logging
logging.config.fileConfig('log.ini', disable_existing_loggers=False)
logger = logging.getLogger(__name__)  # the __name__ resolve to "main" since we are at the root of the project. 
                                      # This will get the root logger since no logger in the configuration has this name.

# import the language codes
try:
    with open(LANGS, 'r') as f:
        languages = json.load(f)
        logger.info(languages)
except FileNotFoundError:
    logger.error("Could not find languages.json")
    sys.exit(1)

# ==== Setup ==== #
app = FastAPI()

# ===  CORS === #
origins = ["*"]
app.add_middleware(
	CORSMiddleware,
	allow_origins=origins,
	allow_credentials=True,
	allow_methods=["*"],
	allow_headers=["*"],
)

# === Request Models === #
class STTRequest(BaseModel):
    sample: str
    use_decoder: bool

# === ML Models === #
models = {}
decoders = {}
for code, lang in languages.items():
	models[code] = SpeechRecognitionModel("jonatasgrosman/wav2vec2-large-xlsr-53-" + lang)
	lm_path = "../files/" + code + "/lm.binary"
	unigrams_path = "../files/" + code + "/unigrams.txt"
	decoders[code] = KenshoLMDecoder(models[code].token_set, lm_path=lm_path, unigrams_path=unigrams_path)


@app.post('/transcribe_audio/{language}')
def transcribe_audio(language: str, data: STTRequest):
    '''
    Safes a single audio sample temporarily and transcribes it.

    expects the following json:
    {
        "sample": "base64 encoded wav file" (string)
		"use_decoder": bool
    }

    on success returns the following json:
    {
		TODO
    }
    '''

    # check if the language is valid
    language_code = language
    if language_code not in languages:
        return HTTPException(status_code=400, detail="Invalid language code")

    sample = data.sample
    use_decoder = data.use_decoder

    # check if the sample is valid
    if not isinstance(sample, str):
        raise HTTPException(status_code=400, detail="Invalid sample")

    sample_path = "file"

    try:
        decodedData = base64.b64decode(sample)
        with open(sample_path, 'wb') as file:
            file.write(decodedData)

    except Exception as e:
        logger.exception(e)
        raise HTTPException(status_code=500, detail="Could not write sample to file")
        
    try:
        if use_decoder:
            transcriptions = models[language_code].transcribe([sample_path], decoder=decoders[language_code])
        else:
            transcriptions = models[language_code].transcribe([sample_path])

        logger.info(transcriptions)
        return transcriptions

    except Exception as e:
        logger.exception(e)
        raise HTTPException(status_code=500, detail="Could not process sample")


