'''
This script creates the endpoints for the API.

@ Author: Paul Bethge
@ Date: 01/09/2022

Copyright (c) 2022 ZKM | Karsruhe
Simplified BSD License
'''

# Imports
import os
import sys
import json
import time
import base64
import logging
import logging.config as cnfg
import uuid

from pydantic import BaseModel
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware

import whisper
from transcribe import transcribe 


# ==== Configuration ==== #

 # configure logging
logging.config.fileConfig('log.ini', disable_existing_loggers=False)
logger = logging.getLogger(__name__)  # the __name__ resolve to "main" since we are at the root of the project. 
                                      # This will get the root logger since no logger in the configuration has this name.
# ==== Setup ==== #
app = FastAPI()

# ===  CORS === #
origins = ["*"]
app.add_middleware(
	CORSMiddleware,
	allow_origins=origins,
	allow_credentials=True,
	allow_methods=["*"],
	allow_headers=["*"],
)

# === Request Models === #
class STTRequest(BaseModel):
    sample: str

# === ML Models === #
model = whisper.load_model("medium")


async def get_prediction(filename):
    try:
        prediction = transcribe(model, filename)
        return prediction
    except Exception as e:
        print(e)
        raise HTTPException(status_code=500, detail="Could not process sample")


@app.post('/transcribe_audio/')
async def transcribe_audio(data: STTRequest):
    '''
    Safes a single audio sample temporarily and transcribes it.

    expects the following json:
    {
        "sample": "base64 encoded wav file" (string)
    }

    on success returns the following json:
    {
		TODO
    }
    '''

    sample = data.sample

    # check if the sample is valid
    if not isinstance(sample, str):
        raise HTTPException(status_code=400, detail="Invalid sample")

    # get random hash 32 bytes long for a filename
    sample_path = "file" + str(uuid.uuid4()) + str(time.time())
    try:
        decodedData = base64.b64decode(sample)
        with open(sample_path, 'wb') as file:
            file.write(decodedData)

    except Exception as e:
        logger.exception(e)
        raise HTTPException(status_code=500, detail="Could not write sample to file")
        
    try:
        print("Transcribing...")
        prediction = await get_prediction(sample_path)

    except Exception as e:
        logger.exception(e)
        # remove the sample
        os.remove(sample_path)
        raise HTTPException(status_code=500, detail="Could not process sample")

    # remove the sample
    os.remove(sample_path)

    return prediction
